require "util"
local game_ready = true -- init game on first tick as soon as game is ready

-- search exsisting energy things
local function init_energy()
   print("init energy")
   global.energy = {}
   global.energy_sources = {"generator", "solar-panel", "reactor", "accumulator"}
   for _, source in ipairs(global.energy_sources) do
      global.energy[source] = {}
      for _,surf in pairs(game.surfaces) do
         local ents = surf.find_entities_filtered({type = source})
         for _, ent in pairs(ents) do
            global.energy[source][ent.unit_number] = ent--you don't need position, besides that can be changed
         end
      end
   end
end
script.on_init(init_energy)

function add_energy_entity(entity)
   print("adding: " .. entity.type .. ", id: " .. tostring(entity.unit_number))
   print(tostring(global.energy[entity.type]))
   for k, _ in pairs(global.energy) do
	print("  " .. k)
   end
   if global.energy[entity.type] then--you already hava a dictionary, no need to search through it
      global.energy[entity.type][entity.unit_number] = entity
   	print("added: " .. entity.type)
   end
end

function delete_energy_entity(entity)
   if global.energy[entity.type] then
      global.energy[entity.type][entity.unit_number] = nil
   end
end

function get_entitys_energy(source)
   local cur = 0
   local max = 0
   for _, ent in pairs(global.energy[source]) do
      cur = cur + ent.energy
      max = max + ent.electric_buffer_size
   end
   local energy = {cur = cur, max = max}
   return energy
end

function write_stats(filename, stats)
	local str = ""
	for k, v in pairs(stats) do
		str = str .. k .. " " .. tostring(v["cur"]) .. " / " .. tostring(v["max"]).. "\r\n"
	end
	game.write_file("prod_stats/" .. filename, str)
end

function write_energy()
	local energy = {}
	for _, source in ipairs(global.energy_sources) do
		energy[source] = get_entitys_energy(source)
	end
	write_stats("energy.txt", energy)
end


script.on_event(defines.events.on_tick, function(event)
	if game_ready then
		init_energy()
		game_ready = false
	end
	if game.tick % 60 == 0 then
		write_energy()
	end
end)
--
--- Entity Related Events
script.on_event({
	defines.events.on_built_entity, 
	defines.events.on_robot_built_entity
	}, function(event)
	add_energy_entity(event.created_entity)
end)

script.on_event({
	defines.events.on_preplayer_mined_item,
	defines.events.on_robot_pre_mined,
	defines.events.on_entity_died
	}, function(event)
    	delete_energy_entity(event.entity)
end)
